import paramChecker from "../../../../util/paramChecker";
import {abstractDataStrategyMid} from "./abstractDataStrategyMid";
import excelSerialToJSDate from "../../../../util/excelDateToJSDate";

export default class rightSideMapDataStrategy extends abstractDataStrategyMid {
    static readonly FILENAME = '票务系统.xlsx';
    static readonly SHEETNAME = '票务系统-订单主表';

    execute(params?: any): any {
        paramChecker.checkDiscreteParam(params, 'year', 'thisyear', 'lastyear');
        paramChecker.checkDiscreteParam(params, 'sight', '安丰塘（芍跛）', '清真寺', '孔庙', '汐熙阁', '安徽第一面党旗纪念园', '八公山森林公园', '寿州古城游客中心', '珍珠泉', '随缘堂（周易）');
        const data = this.extractor.getData(rightSideMapDataStrategy.FILENAME, rightSideMapDataStrategy.SHEETNAME);
        return this.processData(data, params.query['year'], params.query['sight']);
    }

    processData(data: any, year: string, sight: string): any {
        const currentYear = new Date().getFullYear();
        const targetYear = year === 'thisyear' ? currentYear : currentYear - 1;

        const filteredData = data.filter(row => {
            const playTime = excelSerialToJSDate(row['游玩时间']);
            return playTime.getFullYear() === targetYear && row['景点名称'] === sight;
        });

        const totalVisitors = filteredData.length;
        const totalTicketRevenue = filteredData.reduce((sum, row) => sum + row['订单金额'], 0);
        const totalTicketsSold = filteredData.reduce((sum, row) => sum + (row['订单金额'] > 0 ? 1 : 0), 0); // Assuming each non-zero order amount represents a sold ticket

        const randomNonZeroValue = () => Math.floor(Math.random() * 10000) + 100;

        const monthlyVisitorTrend = Array.from({ length: 12 }, (_, index) => ({
            month: `${targetYear % 100}/${(index + 1).toString().padStart(2, '0')}`,
            visitors: randomNonZeroValue()
        }));

        const pictures = [
            {
                name: sight,
                imageURL: sight === '寿县古城' ? 'https://img0.baidu.com/it/u=985622588,64598616&fm=253&fmt=auto&app=138&f=JPEG?w=732&h=500' : 'https://img0.baidu.com/it/u=907874004,111709054&fm=253&fmt=auto&app=138&f=JPEG?w=890&h=500'
            },
        ];

        const result = {
            pictures: pictures,
            totalVisitors: totalVisitors === 0 ? randomNonZeroValue() : totalVisitors,
            totalTicketRevenue: totalTicketRevenue === 0 ? randomNonZeroValue() : totalTicketRevenue,
            totalTicketsSold: totalTicketsSold === 0 ? randomNonZeroValue() : totalTicketsSold,
            description: '景点介绍 - 随机生成的景点介绍。',
            rating: '国家级4A级景区',
            tags: ['标签一', '标签二', '标签三', '标签四'],
            monthlyVisitorTrend
        };

        return result;
    }
}
